#!/bin/sh

# autoprovision stage 1: this script will be executed upon boot without a valid extroot (i.e. when rc.local is found and run from the internal overlay)

. /root/autoprovision-functions.sh

getPendriveSize()
{
    # this is needed for the mmc card in some (all?) Huawei 3G dongle.
    # details: https://dev.openwrt.org/ticket/10716#comment:4
    if [ -e /dev/sda ]; then
        # force re-read of the partition table
        head -c 1024 /dev/sda >/dev/null
    fi

    if (grep -q sda /proc/partitions) then
        cat /sys/block/sda/size
    else
        echo 0
    fi
}

hasBigEnoughPendrive()
{
    local size=$(getPendriveSize)
    # 6GB minimum
    if [ $size -ge 12288000 ]; then
        log "Found a pendrive of size: $(($size / 2 / 1024)) MB"
        return 0
    else
        return 1
    fi
}

rereadPartitionTable()
{
    log "Rereading partition table"
    blockdev --rereadpt /dev/sda
}

setupPendrivePartitions()
{
    # log "Erasing partition table"
    # # erase partition table
    # dd if=/dev/zero of=/dev/sda bs=1k count=256

    rereadPartitionTable

    log "Creating partitions"
    # sda1 is 'swap'
    # sda2 is 'root'
    # sda3 is 'data', if there's any space left
    sgdisk --zap-all /dev/sda
    sgdisk -n1:1MiB:+128MiB -t1:8200 /dev/sda
    sgdisk -n2:0:+6144MiB -t2:8300 /dev/sda
    sgdisk -n3:0:0 -t3:8300 /dev/sda

    log "Finished partitioning /dev/sda using sgdisk"

    rereadPartitionTable

    until [ -e /dev/sda1 ]
    do
        echo "Waiting for partitions to show up in /dev"
        sleep 1
    done

    mkswap -L swap -U $swapUUID /dev/sda1
    mkfs.f2fs -f -l root -U $rootUUID /dev/sda2
    mkfs.f2fs -f -l data -U $dataUUID /dev/sda3

    log "Finished setting up filesystems"
}

setupExtroot()
{
    mkdir -p /mnt/extroot/
    mount -U $rootUUID /mnt/extroot

    tar -C /overlay -cvf - . | tar -C /mnt/extroot -xf -

    overlay_root=/mnt/extroot/upper

    # at this point we could copy the entire root (a previous version of this script did that), or just the overlay from the flash,
    # but it seems to work fine if we just create an empty overlay that is only replacing the rc.local from the firmware.

    # let's write a new rc.local on the extroot that will shadow the one which is in the rom (to run stage2 instead of stage1)
    mkdir -p ${overlay_root}/etc/
    cat >${overlay_root}/etc/rc.local <<EOF
/root/autoprovision-stage2.sh
exit 0
EOF

    disableStage1

    log "Finished setting up extroot"
}

setupFstab()
{
    log "Modify fstab"

    ORIG="$(block info | sed -n -e '/MOUNT="\S*\/overlay"/s/:\s.*$//p')"

    uci -q delete fstab.swap
    uci -q delete fstab.extroot
    uci -q delete fstab.data
    uci -q delete fstab.rwm

    uci set fstab.@global[0]=global
    uci set fstab.@global[0].anon_swap="0"
    uci set fstab.@global[0].anon_mount="0"
    uci set fstab.@global[0].auto_swap="0"
    uci set fstab.@global[0].auto_mount="0"
    uci set fstab.@global[0].delay_root="10"
    uci set fstab.@global[0].check_fs="0"
    uci set fstab.@swap[0]=swap
    uci set fstab.@swap[0].uuid="${swapUUID}"
    uci set fstab.@swap[0].enabled="1"
    uci set fstab.extroot=mount
    uci set fstab.extroot.uuid="${rootUUID}"
    uci set fstab.extroot.target="/overlay"
    uci set fstab.extroot.fstype="f2fs"
    uci set fstab.extroot.options="rw,noatime"
    uci set fstab.extroot.enabled="1"
    uci set fstab.extroot.enabled_fsck="0"
    uci set fstab.data=mount
    uci set fstab.data.uuid="${dataUUID}"
    uci set fstab.data.target="/mnt/data"
    uci set fstab.data.fstype="f2fs"
    uci set fstab.data.options="rw,noatime"
    uci set fstab.data.enabled="1"
    uci set fstab.data.enabled_fsck="0"
    uci set fstab.rwm=mount
    uci set fstab.rwm.device="${ORIG}"
    uci set fstab.rwm.target="/rwm"
    uci set fstab.rwm.enabled="1"
    uci set fstab.rwm.enabled_fsck="0"
    uci commit fstab

    log "Finished modifying fstab"
}

disableStage1()
{
    # FIXME it would be more future-proof to transform the rc.local file
    # instead of overwriting it.
    cat >/etc/rc.local <<EOF
# If you want to re-run the initialization of the extroot once again,
# then remove the comment from before the stage1.sh script below,
# reboot the router without the extroot pendrive, and when it's booted
# then insert the pendrive. It will then be reformatted and
# reinitialized.

#/root/autoprovision-stage1.sh

exit 0
EOF
}

autoprovisionStage1()
{
    signalAutoprovisionWorking

    signalAutoprovisionWaitingForUser
    signalWaitingForPendrive

    until hasBigEnoughPendrive
    do
        echo "Waiting for a pendrive to be inserted"
        sleep 3
    done

    signalAutoprovisionWorking # to make it flash in sync with the USB led
    signalFormatting

    sleep 1

    setupPendrivePartitions
    sleep 1
    setupExtroot

    # setupFstab

    sync
    stopSignallingAnything
    reboot
}

autoprovisionStage1
